unit NTSrvExp;

//------------------------------------------------------------------------------
// With this expert 'Delphi 4 Professional' offers a simple and powerful way
// to create win32 service applications. Use this expert as a starting point,
// and use the online help as guide.
//
// To install this expert, you must first start 'Delphi 4 Professional'.
// Install this unit (component) in a new or existing packages. After
// this the expert will appear in the register 'New' of the object repository.
//
// To install a service, launch the program from the commandline with the
// parameter /install. Example: project1 /install
//
// D-45881 Gelsenkirchen, July 1999
//                                                              Christian Storb
//                                                            StorbChri@aol.com
// -----------------------------------------------------------------------------

interface

uses
  EditIntf, ExptIntf, ToolIntf, Windows;

type
  TNTServiceApplicationExpert = class(TIExpert)
  public
    procedure Execute; override;
    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    function GetName: string; override;
    function GetPage: string; override;
    function GetState: TExpertState; override;
    function GetStyle: TExpertStyle; override;
  end;

  TNTServiceExpert = class(TIExpert)
  public
    procedure Execute; override;
    function GetAuthor: string; override;
    function GetComment: string; override;
    function GetGlyph: HICON; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    function GetName: string; override;
    function GetPage: string; override;
    function GetState: TExpertState; override;
    function GetStyle: TExpertStyle; override;
  end;

procedure Register;

implementation

{$R *.res}

uses
  Classes, DsgnIntf, SvcMgr, SysUtils;

resourcestring
  ProjectSource =
  'program %s; '                                                      + #13#10 +
  ''                                                                  + #13#10 +
  'uses '                                                             + #13#10 +
  '  SvcMgr;'                                                         + #13#10 +
  ''                                                                  + #13#10 +
  '{$R *.RES}'                                                        + #13#10 +
  ''                                                                  + #13#10 +
  'begin'                                                             + #13#10 +
  '  Application.Initialize;'                                         + #13#10 +
  '  Application.Run;'                                                + #13#10 +
  'end.';

  ModuleSource =
  'unit %0:s;'                                                        + #13#10 +
  ''                                                                  + #13#10 +
  'interface'                                                         + #13#10 +
  ''                                                                  + #13#10 +
  'uses'                                                              + #13#10 +
  '  Windows, Messages, SysUtils, Classes, Graphics, Controls, ' +
     'SvcMgr, Dialogs;'                                               + #13#10 +
  ''                                                                  + #13#10 +
  'type'                                                              + #13#10 +
  '  T%1:s = class(T%2:s)'                                            + #13#10 +
  '  private'                                                         + #13#10 +
  '    { private declarations } '                                     + #13#10 +
  '  public '                                                         + #13#10 +
  '    { public declarations }'                                       + #13#10 +
  '    function GetServiceController: PServiceController; override; ' + #13#10 +
  '  end;'                                                            + #13#10 +
  ' '                                                                 + #13#10 +
  'var '                                                              + #13#10 +
  '  %1:s: T%1:s;'                                                    + #13#10 +
  ''                                                                  + #13#10 +
  'implementation '                                                   + #13#10 +
  ' '                                                                 + #13#10 +
  '{$R *.DFM}'                                                        + #13#10 +
  ' '                                                                 + #13#10 +
  'procedure ServiceController(CtrlCode: DWord); stdcall; '           + #13#10 +
  'begin '                                                            + #13#10 +
  '  %1:s.Controller(CtrlCode); '                                     + #13#10 +
  'end; '                                                             + #13#10 +
  ' '                                                                 + #13#10 +
  'function T%1:s.GetServiceController: PServiceController;'          + #13#10 +
  'begin'                                                             + #13#10 +
  '  Result := @ServiceController; '                                  + #13#10 +
  'end;'                                                              + #13#10 +
  ''                                                                  + #13#10 +
  'end.';

type
  TNTServiceProjectCreator = class(TIProjectCreator)
  public
    function Existing: Boolean; override;
    function GetFileName: String; override;
    function GetFileSystem: String; override;
    function NewProjectSource(const ProjectName: String): String; override;
    procedure NewDefaultModule; override;
    procedure NewProjectResource(Module: TIModuleInterface); override;
  end;

  TNTServiceModuleCreator = class(TIModuleCreator)
  public
    function Existing: Boolean; override; stdcall;
    function GetAncestorName: string; override; stdcall;
    function GetFileName: string; override; stdcall;
    function GetFileSystem: string; override; stdcall;
    function GetFormName: string; override; stdcall;
    function NewModuleSource(const UnitIdent, FormIdent,
      AncestorIdent: string): string; override; stdcall;
    procedure FormCreated(Form: TIFormInterface); override; stdcall;
  end;

//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterCustomModule(TService, TCustomModule);
  RegisterLibraryExpert(TNTServiceApplicationExpert.Create);
  RegisterLibraryExpert(TNTServiceExpert.Create);
end;


{ TNTServiceApplicationExpert }

//------------------------------------------------------------------------------
procedure TNTServiceApplicationExpert.Execute;
var
  ProjectCreator  : TNTServiceProjectCreator;
  ModuleInterface : TIModuleInterface;
begin
  ProjectCreator := TNTServiceProjectCreator.Create;
  try
    ModuleInterface := ToolServices.ProjectCreate(ProjectCreator, []);
    ModuleInterface.Free;
  finally
    ProjectCreator.Free;
  end;
end;

//------------------------------------------------------------------------------
// This should return the "author" of this expert. This could be a person or
// company, for example.  This value will be displayed in the Object
// Repository.
function TNTServiceApplicationExpert.GetAuthor: string;
begin
  Result := 'Christian Storb';
end;

//------------------------------------------------------------------------------
// This should return a 1 - 2 sentence describing the function of this expert.
function TNTServiceApplicationExpert.GetComment: string;
begin
  Result := 'Creates a win32-systemservice';
end;

//------------------------------------------------------------------------------
// This should return a handle to a icon to be displayed in the form or
// project list boxes or dialogs.  Return 0 to display the default icon.
function TNTServiceApplicationExpert.GetGlyph: HICON;
begin
  Result := LoadIcon(hInstance, 'NTSERVICEAPP');
end;

//------------------------------------------------------------------------------
// This ID string should be unique to all experts that could be installed.
// By convention, the format of the string is: CompanyName.ExpertFunction,
// ex. Borland.WidgetExpert
function TNTServiceApplicationExpert.GetIDString: string;
begin
  Result := 'ChristianStorb.ServiceApplication';
end;

//------------------------------------------------------------------------------
// This must return a unique descriptive name identifying this expert.
function TNTServiceApplicationExpert.GetName: string;
begin
  Result := 'Service-Application'
end;

//------------------------------------------------------------------------------
// Should return short string indicating on  which page in the repository
// this expert should be placed.
// NOTE: The user can still override this setting from the Tool|Repository
// dialog.
function TNTServiceApplicationExpert.GetPage: string;
begin
  Result := 'New';
end;

//------------------------------------------------------------------------------
// Returns one of four possible values:
//                 esStandard - Tells the IDE to treat the interface to
//                              this expert as a menu item on the Tools/Experts
//                              menu.
//                 esForm     - Tells the IDE to treat this expert interface
//                              in a fashion similar to form templates.
//                 esProject  - Tells the IDE to treat this interface in a
//                              fashion similar to project templates.
//                 esAddIn    - Tells the IDE that this expert handles all its
//                              own interfacing to the IDE through the
//                              TIToolServices interface.
function TNTServiceApplicationExpert.GetStyle: TExpertStyle;
begin
  Result := esProject;
end;

//------------------------------------------------------------------------------
// This should return the actual text to display for the menu item.
// NOTE: This function is called each time the parent menu is pulled-down,
// so it is possible to provide context sensative text.
function TNTServiceApplicationExpert.GetMenuText: string;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
// If the style is esStandard, esChecked will cause the menu to display a
// checkmark.
// NOTE: This function is called each time the expert is shown in a menu or
// listbox in order to determine how it should be displayed.
function TNTServiceApplicationExpert.GetState: TExpertState;
begin
  Result := []
end;


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


{ TNTServiceExpert }

//------------------------------------------------------------------------------
procedure TNTServiceExpert.Execute;
var
  ModuleCreator   : TNTServiceModuleCreator;
  ModuleInterface : TIModuleInterface;
begin
  ModuleCreator := TNTServiceModuleCreator.Create;
  try
    ModuleInterface := ToolServices.ModuleCreate(ModuleCreator, [cmAddToProject,
      cmShowSource, cmShowForm, cmUnNamed, cmMarkModified]);
    ModuleInterface.Free;
  finally
    ModuleCreator.Free;
  end;
end;

//------------------------------------------------------------------------------
// This should return the "author" of this expert. This could be a person or
// company, for example.  This value will be displayed in the Object
// Repository.
function TNTServiceExpert.GetAuthor: string;
begin
  Result := 'Christian Storb';
end;

//------------------------------------------------------------------------------
// This should return a 1 - 2 sentence describing the function of this expert.
function TNTServiceExpert.GetComment: string;
begin
  Result := 'Creates a win32-service';
end;

//------------------------------------------------------------------------------
// This should return a handle to a icon to be displayed in the form or
// project list boxes or dialogs.  Return 0 to display the default icon.
function TNTServiceExpert.GetGlyph: HICON;
begin
  Result := LoadIcon(hInstance, 'NTSERVICE');
end;

//------------------------------------------------------------------------------
// This ID string should be unique to all experts that could be installed.
// By convention, the format of the string is: CompanyName.ExpertFunction,
// ex. Borland.WidgetExpert
function TNTServiceExpert.GetIDString: string;
begin
  Result := 'ChristianStorb.Service';
end;

//------------------------------------------------------------------------------
// This should return the actual text to display for the menu item.
// NOTE: This function is called each time the parent menu is pulled-down,
// so it is possible to provide context sensative text.
function TNTServiceExpert.GetMenuText: string;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
// This must return a unique descriptive name identifying this expert.
function TNTServiceExpert.GetName: string;
begin
  Result := 'Service';
end;

//------------------------------------------------------------------------------
// Should return short string indicating on  which page in the repository
// this expert should be placed.
// NOTE: The user can still override this setting from the Tool|Repository
// dialog.
function TNTServiceExpert.GetPage: string;
begin
  Result := 'New';
end;

//------------------------------------------------------------------------------
// If the style is esStandard, esChecked will cause the menu to display a
// checkmark.
// NOTE: This function is called each time the expert is shown in a menu or
// listbox in order to determine how it should be displayed.
function TNTServiceExpert.GetState: TExpertState;
begin
  Result := [];
end;

//------------------------------------------------------------------------------
// Returns one of four possible values:
//                 esStandard - Tells the IDE to treat the interface to
//                              this expert as a menu item on the Tools/Experts
//                              menu.
//                 esForm     - Tells the IDE to treat this expert interface
//                              in a fashion similar to form templates.
//                 esProject  - Tells the IDE to treat this interface in a
//                              fashion similar to project templates.
//                 esAddIn    - Tells the IDE that this expert handles all its
//                              own interfacing to the IDE through the
//                              TIToolServices interface.
function TNTServiceExpert.GetStyle: TExpertStyle;
begin
  Result := esForm;
end;


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


{ TNTServiceProjectCreator }

//------------------------------------------------------------------------------
function TNTServiceProjectCreator.Existing: Boolean;
begin
  Result := False;
end;

//------------------------------------------------------------------------------
function TNTServiceProjectCreator.GetFileName: String;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
function TNTServiceProjectCreator.GetFileSystem: String;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
procedure TNTServiceProjectCreator.NewDefaultModule;
var
  ModuleCreator   : TNTServiceModuleCreator;
  ModuleInterface : TIModuleInterface;
begin
  ModuleCreator := TNTServiceModuleCreator.Create;
  try
    ModuleInterface := ToolServices.ModuleCreate(ModuleCreator, [cmMainForm,
      cmAddToProject, cmShowSource, cmShowForm, cmMarkModified, cmUnNamed]);
    ModuleInterface.Free;
  finally
    ModuleCreator.Free;
  end;
end;

//------------------------------------------------------------------------------
procedure TNTServiceProjectCreator.NewProjectResource(Module: TIModuleInterface);
begin
  Module.Free;
end;

//------------------------------------------------------------------------------
function TNTServiceProjectCreator.NewProjectSource(
  const ProjectName: String): String;
begin
  Result := Format(ProjectSource, [ProjectName]);
end;


//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------


{ TNTServiceModuleCreator }

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.Existing: Boolean;
begin
  Result := False;
end;

//------------------------------------------------------------------------------
procedure TNTServiceModuleCreator.FormCreated(Form: TIFormInterface);
begin
  Form.Free;
end;

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.GetAncestorName: String;
begin
  Result := 'Service';
end;

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.GetFileName: String;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.GetFileSystem: String;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.GetFormName: String;
begin
  Result := EmptyStr;
end;

//------------------------------------------------------------------------------
function TNTServiceModuleCreator.NewModuleSource(const UnitIdent, FormIdent,
  AncestorIdent: String): String;
begin
  Result := Format(ModuleSource, [UnitIdent, FormIdent, AncestorIdent]);
end;

end.
